# /etc/nixos/flake.nix
{
  description = "flake for mira::{auxin, leptin, gastrin}";

  inputs = {
    mirafetch.url = "github:ArgentumCation/mirafetch";
    nixpkgs.url = "github:NixOS/nixpkgs/nixos-24.05";
    nixos-unstable = {
      url = "github:NixOS/nixpkgs/nixos-unstable";
      # inputs.nixpkgs.follows = "nixos-unstable";
    };
    nixpkgs-unstable = {
      url = "github:NixOS/nixpkgs/nixpkgs-unstable";
      # inputs.nixpkgs.follows = "nixpkgs-unstable";
    };
    nixpkgs-darwin.url = "github:NixOS/nixpkgs/nixpkgs-24.05-darwin";
    neovim-nightly-overlay.url = "github:nix-community/neovim-nightly-overlay";
    nix-darwin = {
      url = "github:LnL7/nix-darwin";
      # inputs.nixpkgs.follows = "nixpkgs-darwin";
    };
  };

  outputs = {
    self,
    nixos-unstable,
    nixpkgs-darwin,
    nix-darwin,
    nixpkgs-unstable,
    ...
  } @ inputs: let
    overlays = [inputs.neovim-nightly-overlay.overlays.default];
  in {
    nixosConfigurations = {
      auxin = nixos-unstable.lib.nixosSystem {
        system = "x86_64-linux";
        specialArgs = {
          meta = {hostname = "auxin";};
        };
        modules = [
          ./auxin/configuration.nix
          ./auxin/hardware-configuration.nix
          ./common.nix
          # ./primary.nix
        ];
      };
      orexin = nixos-unstable.lib.nixosSystem {
        system = "x86_64-linux";
        specialArgs = {
          meta = {hostname = "orexin";};
        };
        modules = [
          ./common.nix
          ./orexin/configuration.nix
          ./orexin/hardware-configuration.nix
        ];
      };
      galanin = nixos-unstable.lib.nixosSystem {
        system = "x86_64-linux";
        specialArgs = {
          meta = {hostname = "galanin";};
        };
        modules = [
          ./common.nix
          ./galanin/configuration.nix
          ./galanin/hardware-configuration.nix
        ];
      };
      gastrin = nixos-unstable.lib.nixosSystem {
        system = "x86_64-linux";
        specialArgs = {
          meta = {hostname = "gastrin";};
        };
        modules = [
          ./gastrin/configuration.nix
          ./common.nix
        ];
      };
    };
    darwinConfigurations = {
      leptin = nix-darwin.lib.darwinSystem {
        system = "aarch64-darwin";
        modules = [
          ./leptin/configuration.nix
          ./common.nix
          {nixpkgs.overlays = overlays;}
        ];
        specialArgs = {
          meta = {hostname = "leptin";};
          inherit nixpkgs-darwin;
          inherit nix-darwin;
          inherit inputs;
          inherit nixpkgs-unstable;
        };
      };
    };
  };
}
